#ifndef DIRECTORY_H
#define DIRECTORY_H

#include <openfile.h>
#include "synch.h"

#define FileNameMaxLen 		14

class DirectoryEntry {
public:			 	// public, for convenience
    bool inUse;
    int sector;				// where to find the FileHeader for name
    char name[FileNameMaxLen + 1];	// +1 for the trailing '\0'
};

const int DIR_ENTRY_SIZE = sizeof (DirectoryEntry);
const int DIR_2_ENTRIES_SIZE = sizeof (DirectoryEntry) * 2;

class Directory {
public:
    Directory(OpenFile *dirFile);
    ~Directory();

    bool Initialize(int dotdot);

    Directory *Clone();

    int Find(char *name);		// find the file header for "name"
    bool Add(char *name, int newSector);// add name into the directory.
    bool Remove(char *name);		// remove name from directory.
					// if "." is removed, the directory
					// itself will be removed.
					// the associated file is removed, also.

    void DirReadLock() { dirFilePtr->DirReadLock(); }
    void DirReadUnlock() { dirFilePtr->DirReadUnlock(); }
    void DirWriteLock() { dirFilePtr->DirWriteLock(); }
    void DirWriteUnlock() { dirFilePtr->DirWriteUnlock(); }

    // returns TRUE if directory has been deleted.
    bool Deleted();

    // returns TRUE if file header sector 'sector' can be removed.
    bool CheckRemove(int sector);

    int GetSector();

    void List(int indent, bool verbose);
private:
    OpenFile *dirFilePtr;
};

#endif
