// diskbitmap.h -- class for managing a bitmap for disk blocks.  
// Initialized with the number of bits being managed

#ifndef DISKBITMAP_H
#define DISKBITMAP_H

#include "utility.h"
#include <openfile.h>

#define BitsInByte 	8
#define BitsInWord 	32

#include "synch.h"

class DiskBitMap {
public:

    DiskBitMap(int nitems, int sector, bool initialize);
    ~DiskBitMap();
    
    void Mark(int which);   	// set bit
    void Clear(int which);  	// clear bit
    bool Test(int which);   	// is bit set?
    int Find();            	// return a bit that is clear, -1 if none
    int NumClear();		// return the number of clear bits

    void Print();		// print contents of bitmap
    void Update();

private:
    int sectorNumber;		// need to remember sector # for updates.
    bool dirty;			// is bitmap dirty ?
    Lock *mutex;		// for mutual exclusion.
    int numBits;
    unsigned int *map;
};

#endif
