// filesys.h -- Class representing the file system.
// ----------------------------------------------------------------------------
// NOTE: All code is changed, so we didn't use any #ifdef in this file.
// Also blew away a lot of comments  that don't make sense now
// ----------------------------------------------------------------------------

#include "copyright.h"

#ifndef FS_H
#define FS_H

#include "utility.h"

class Lock;
class OpenFile;
class Directory;

#define TransferSize    10      // make it small, just to be difficult

class FileSystem {
public:
    // Initialize the file system -- assumes "synchDisk" has been initialized.
    // If format is set, the disk is garbage, so initialize the directory
    // and the map of free blocks.
    FileSystem(bool format);
    ~FileSystem();

    OpenFile *Create(Directory *currDir, char *name);	// create a file
    OpenFile *Open(Directory *currDir, char *name); 	// open a file

    bool Remove(Directory *currDir, char *name); // delete a file (UNIX unlink)

    Directory *CloneRootDir();		// clone root directory,
					// for the 1st process.
    Directory *CloneDir(Directory *dir);// clone a directory, for Exec().
    Directory *ChDir(Directory *currDir, char *name);
    bool MkDir(Directory *currDir, char *name);
    void Sync(); // unlike UNIX sync, this is synchronous.

    // debug
    void List();
    void Print();
    void Stats();
    bool Init();

 public:
    Directory *m_RootDirectory; // for convienience

 private:
   // common code for Create(), Open(), Mkdir()
   enum op_t { OP_CREATE, OP_OPEN, OP_CHDIR, OP_MKDIR, };
   bool CommonOpen(Directory *, char *, op_t operation, OpenFile **of);
   bool Copy(char *from,char *to);
   // this method follows a pathname 'path', starting from directory 'start'.
   // If successful, this method returns TRUE, and 'dir' will be initialized
   // with the last directory, while 'lastComponent' will contain the last part
   // of the pathname.
   bool FollowPath(Directory *start, char *path,
		   Directory ** dir, char *lastComponent);
   Lock *mutex;
};

#endif
