// openfile.h -- 
// NOTE: All code is changed, so we didn't use #ifdef CHANGED in this file.
// orignal comments at the top are inappropriate now, so we removed them

#ifndef OPENFILE_H
#define OPENFILE_H

class FileHeader;
#include "syscall.h"

typedef struct filestat {
  int fileSize;       /* file size, in bytes */
  int fileBlocks;     /* # of blocks, including indirect pointers */
  int fileType;       /* file type */
  int fileSector;     /* file sector */
} filestat_t;

#define FSTAT_TYPE_REG 1
#define FSTAT_TYPE_DIR 2

class OpenFile {
public:
    OpenFile(int sector, bool useReservedSlots = FALSE); // open the file
    ~OpenFile();			// close the file

    // set the position to start reading/writing -- like UNIX lseek
    void Seek(int position) {
	m_SeekPosition = position;
	m_Sequential = FALSE;
    }

    // Read/write bytes from the file; return the # actually read/written.
    int Read(char *into, int numBytes);
    int Write(char *from, int numBytes);

    // Bypass call to Seek
    int ReadAt(char *into, int numBytes, int position, bool isSeq = FALSE);
    int WriteAt(char *into, int numBytes, int position, bool isSeq = FALSE);

    void FileReadLock();
    void FileReadUnlock();
    void FileWriteLock();
    void FileWriteUnlock();
    void DirReadLock();
    void DirReadUnlock();
    void DirWriteLock();
    void DirWriteUnlock();

    void SetBusy();	// mark file busy -- this is done on binary/swap files
			// by addrspace, so that no one can write onto them
			// (although they can be removed).
    void MarkDeleted();
    bool IsDeleted();
    bool IsBusy();
    bool Truncate();
    bool IsDir();
    int Position() { return (m_SeekPosition); };
    int GetSector();
    void Stat(filestat_t *stat);
    bool Constructed() { return (m_Hdr != NULL ? TRUE : FALSE); };

    int Length(); // simpler than UNIX (lseek to end of file, tell, lseek) 
    
private:
    FileHeader *m_Hdr;
    int m_SeekPosition;
    bool m_Busy;
    bool m_Sequential; // for read-ahead detection.
};

#endif
