// fileio.h - per user file i/o

#ifndef FILEINFO_H
#define FILEINFO_H

#include "syscall.h"
#include "main.h"
#include "utility.h"
#include "openfile.h"

class OpenFile;

#ifdef P4_1
#include "directory.h"
#endif

/************************************************
intended for use by process.
maintains a table of openfiles by the process.
each thread should maintain it's own table
when the thread dies, the table and all the openfile-entries 
in the table will be deleted
when a file is created using Create() syscall, 
the file is closed immediately within FileSystem::Create(), 
and consequently, this table does not add/delete entries upon creation
************************************************/

class FileIO {

public:

#ifdef P4_1
  FileIO(Directory *cDir);
#else
  FileIO();
#endif

    // destructor is called on exit, so it has to close all still-opened
    // files (if any).

    ~FileIO();
    OpenFileId FileOpen(char *filename);
    bool FileClose(OpenFileId fd);
    int FileWrite(OpenFileId fd, char *bp, int nbytes);
    int FileRead(OpenFileId fd, char *bp, int nbytes);

#ifdef P3_3
    int FileLength(OpenFileId fd);
    void FileSeek(OpenFileId fd, int position);
#endif

#ifdef P4_1
    OpenFileId FileCreate(char *filename);
    int FileRemove(char *path); // both file and directories
    Directory *GetCurrentDir() { return (currentDir); };
    int FileStat(OpenFileId fd, filestat_t *stat);
    int ChDir(char *path);
    int MkDir(char *path);
    char * GetPwd() { return pwd ; }
#endif

private:
    const int NFILES = 10;    // max # of open files, per process
    const int MAX_IO_SIZE = 32 * 1024;    // max size for read/write sysc
    struct OpenFileSlot {
    	OpenFileId fd;
      char *name;
      OpenFile *file;
    } openFileSlot[NFILES];
#ifdef P4_1
    Directory *currentDir;
    char pwd[256];
#endif
};

#endif
