#ifdef P3_1
#include "iptmanager.h"
#include "hash.h"
#include "addrspace.h"

class AddrSpace;

static IPTIndex ExtractKeyFromIPTIndex(TEWithPid te) {
  IPTIndex retval(te.virtualPage, te.pid);
  return retval;
}

static unsigned int MyKeyToTEWithPid(IPTIndex ipti){ return ipti.pid; }

static IPTIndex ExtractKeyFromSwapEntry(SwapEntry se) {
  IPTIndex  retval(se.vpn, se.proc);
  return retval;
}

static unsigned int MyKeyToSwapEntry(IPTIndex ipti){return ipti.pid;}

static int IntCompare(int x,int y) {
  if (x < y) return -1;
  if (x > y) return 1;
  return 0;
}

#ifdef P3_2
IPTManager::IPTManager(MemoryManager* mmgrPtr)  {
#else
IPTManager::IPTManager()  {
#endif

  constructed = FALSE;
  mmgr = mmgrPtr;
  ipt = new HashTable<IPTIndex, TEWithPid>(ExtractKeyFromIPTIndex,
					   MyKeyToTEWithPid);
  swapt = new HashTable<IPTIndex, SwapEntry>(ExtractKeyFromSwapEntry,
					   MyKeyToSwapEntry);
  //char* swapfile = "./nachos.swap";

#ifdef P4_1
  swapdir = new Directory(new OpenFile(kernel->fileSystem->m_RootDirectory->Find("swapdir")));
  swapFd = kernel->fileSystem->Create(swapdir,SWAPFILE);
  if(swapFd == NULL) {
#else
  if(!kernel->fileSystem->Create(SWAPFILE)) { 
#endif
    printf("IPTManager: Could not create swap file\n");
    return;
  }
  
#ifdef P4_1

  /*if(!swapdir->Add(SWAPFILE,swapFd->GetSector())) {
    printf("IPTManager: Could not add swap file to swapdir\n");
    return;
    }*/

  swapFd = kernel->fileSystem->Open(swapdir,SWAPFILE);
  if(swapFd == NULL) {
#else
  swapFd = kernel->fileSystem->Open(SWAPFILE);
  if(!swapFd) {
#endif
    printf("IPTManager: Could not open swap file\n");
    return;
  }

  freeBlocksList = new SortedList<int>(IntCompare);
  maxpos=0; //in case, there is not free block create a new one at this position
  //printf("IPTManager::IPTManager() - constructed\n");
  constructed = TRUE;

}

IPTManager::~IPTManager()  {

#ifdef P3_3 

  HashIterator<IPTIndex,TEWithPid> *hi_ipt =
    new HashIterator<IPTIndex,TEWithPid>(ipt);
  List<IPTIndex> *list_ipt = new List<IPTIndex>;
  
  while (!hi_ipt->IsDone()) {
    list_ipt->Append(ExtractKeyFromIPTIndex(hi_ipt->Item()));
    hi_ipt->Next();
  }

  while (!list_ipt->IsEmpty())
    delete &ipt->Remove(list_ipt->RemoveFront());

  delete hi_ipt;
  delete list_ipt;

  HashIterator<IPTIndex,SwapEntry> *hi_swapt =
    new HashIterator<IPTIndex,SwapEntry>(swapt);
  List<IPTIndex> *list_swapt = new List<IPTIndex>;

  while (!hi_swapt->IsDone()) {
    list_swapt->Append(ExtractKeyFromSwapEntry(hi_swapt->Item()));
    hi_swapt->Next();
  }

  while (!list_swapt->IsEmpty())
    delete &swapt->Remove(list_swapt->RemoveFront());

  delete hi_swapt;
  delete list_swapt;

#endif /* P3_3 */

  delete ipt;
  delete swapt;
  delete swapFd;
  delete freeBlocksList;

#ifdef P4_1
  delete swapdir;
#endif
}

bool IPTManager::HandleTlbMiss(int proc, int vpn, TEWithPid *tePtr)
{
  //for now assume that you will find the page in main memory
  IPTIndex ipti(vpn, proc);
  TEWithPid te;
  if(!ipt->Find(ipti, &te)){//Not found in Main Memory
    //printf("INFO!! IPTManager::HandleTlbMiss(proc=%d, vpn=%d)- Did Not Find Page in Main Memory\n", proc, vpn);
    int ppn;
    if(!LoadVPInMemory(kernel->pidManager->getThread(proc)->space, vpn, &ppn)) {
      //printf("ERROR!! IPTManager::HandleTlbMiss(proc=%d, vpn=%d)- Did Not Find Page in Disk Either\n", proc, vpn);
      ASSERT(FALSE);
      return FALSE;
    }
    else{
      te.pid = proc;
      te.virtualPage = vpn;
      te.physicalPage = ppn;
      te.valid = TRUE;
      te.readOnly = FALSE;
      te.use = TRUE;
      te.dirty = FALSE;

    }
  }
  //At this point te has values loaded
  *tePtr = te;
  //printf("INFO!! IPTManager::HandleTlbMiss(proc=%d, vpn=%d)- returning ppn=%d\n", proc, vpn, tePtr->physicalPage);
  return TRUE;
//  return te;
}
bool IPTManager::LoadVPInMemory(AddrSpace* addrsp, int vpn, int *ppnPtr)
{
/* moves a page from phys-mem/swap to phys-mem */
/* may need to move another page from phys-mem to swap */
/*********************************
** real implementation:
** don't forget to add the new page to ipt
** you may need to remove an existing page and save it to disk
**********************************/
  int pid = addrsp->PID();
  //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d)\n", pid, vpn);
  IPTIndex index(vpn, pid);
  TEWithPid te;
  SwapEntry se;
  if(ipt->Find(index, &te) ) {
    *ppnPtr = te.physicalPage;
    /* if the page needs to be loaded
    ** it's definitely not in TLB!
    */
    //return HandleTlbMiss(pid, vpn, &te);
    *ppnPtr = te.physicalPage;
    //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d)i - found ppn=%d in mainmemory\n", pid, vpn, *ppnPtr);
    return TRUE;
  }
  /** page not in phys mem. 
  *** need to get it from swap file
  *** if the swap file does not have it, report error
  *** if found, make enough room in phys mem
  *** Look if a free page available in MemoryManager::freePageList 
  **/
  if(!swapt->Find(index, &se) ) {
    //not in swap file either.
    //printf("ERROR!In IPTManager::LoadVPInMemory(pid=%d, vpn=%d) - Did Not find in swap\n", pid, vpn);
    return FALSE;
  }
  /** page found in swap
  *** to bring it in mainMem, either a free page has to be available in mainMem
  *** or some page in phys-mem will have to be exchanged for this page
  *** if free page avail in mainMem
  ***    update memoryManager->freePageList
  ***    update IPT
  ***    update process pagetable
  ***    update swaptable
  ***    update freeBlocksList
  ***    copy the page to memory
  ***    update ppnPtr
  *** else
  ***    GetFallGuy
  ***    Exchange fallguy for this page
  ***/

  if(!kernel->memoryManager->freePageList->IsEmpty()){
    VirtPage *vPage = (VirtPage *)kernel->memoryManager->freePageList->RemoveFront();
    *ppnPtr = vPage->physnum;
    //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - found replacement candidate in freePageList at ppn=%d\n", pid, vpn, *ppnPtr);
    //need to add it ipt
    te.pid = pid;
    te.virtualPage = vpn;
    te.physicalPage = *ppnPtr;
    te.valid = TRUE;
    ipt->Insert(te);
    //update process pageTable
    addrsp->m_PageTable->Enter(vpn,  *ppnPtr);
    //update swapTable
    se = swapt->Remove(index);
    //update freeBlocksList
    freeBlocksList->Insert(se.position);
    //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - found ppn=%d in swap\n", pid, vpn, se.position);
    return ReadBlockAt(se) && HandleTlbMiss(pid, vpn, &te);;
  }
  else {
    //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - Need to find replacement candidate in mainMemory\n", pid, vpn);
    //need to add it ipt
    TEWithPid toDisk;
    IPTIndex fgi;
    SwapEntry fgs;
    if(!GetFallGuy(&toDisk)){
      //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - could not find replacement candidate in mainmemory\n", pid, vpn);
    //need to add it ipt
      return FALSE;
    }
    fgi.vpn = toDisk.virtualPage;
    fgi.pid = toDisk.pid;
    AddrSpace* fallGuySpace = kernel->pidManager->getThread(toDisk.pid)->space;
    //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - found replacement candidate in mainMemory at ppn=%d\n", pid, vpn, toDisk.physicalPage);
    fallGuySpace->m_PageTable->Remove(toDisk.virtualPage);
    fgs.vpn = toDisk.virtualPage;
    fgs.proc = toDisk.pid;
    fgs.position = GetFreePosition();
    swapt->Insert(fgs);
    //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - replacement candidate ppn=%d will be written to swap at position=%d\n", pid, vpn,  toDisk.physicalPage, fgs.position);
    if(WriteBlockAt(fgs)){;//This looks into ipt to get fileoffset
      //if this page is in TLB, you need to invalidate this entry
      //this is a problem only if the fallguy also belongs to this process
      if(pid == fallGuySpace->PID()){
        TranslationEntry tlbi;
        for(int i = 0; i < TLBSize; i++){
  	  tlbi = kernel->machine->tlb[i];
  	  if(tlbi.virtualPage == toDisk.virtualPage){
            //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - removing TLB entry for replacement candidate ppn=%d \n", pid, vpn, tlbi.physicalPage);
    //need to add it ipt
  	    kernel->machine->tlb[i].valid = FALSE;
  	    break;
  	  }
        }
      }
      ipt->Remove(fgi);//this must be done after write
    }
    else {
      //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - could not write to swap at position=%d \n", pid, vpn, fgs.position);
      return FALSE;
    }
    *ppnPtr = toDisk.physicalPage;
    //need to add it ipt
    te.pid = pid;
    te.virtualPage = vpn;
    te.physicalPage = *ppnPtr;
    te.valid = TRUE;
    ipt->Insert(te);
    //update process pageTable
    addrsp->m_PageTable->Enter(vpn,  *ppnPtr);
    //update swapTable
    se = swapt->Remove(index);
    //printf("In IPTManager::LoadVPInMemory(pid=%d, vpn=%d\n) - Loading at ppn=%d  from swap position==%d\n", pid, vpn, *ppnPtr, se.position);
    freeBlocksList->Insert(se.position);
    return ReadBlockAt(se);
  }
/**********************************
** dummy implementation
**********************************/
  return FALSE;
}
bool IPTManager::FreePageInMemory(AddrSpace* addrsp, int vpn, int *ppnPtr)
{
/*********************************
** unlike LoadVPInMemory, this routine makes a phys-page free
** it may have move the occupying page (if any) to swap
**********************************/
  int pid = addrsp->PID();
  //printf("In IPTManager::FreePageInMemory(pid=%d, vpn=%d) - ", pid, vpn);
  IPTIndex index(vpn, pid);
  TEWithPid te;
  SwapEntry se;
  if(ipt->Find(index, &te) ) {
    //this page already exists in phys-mem
    //you should not erequire a free page to make another copy
    //bail out
    *ppnPtr = te.physicalPage;
    //printf("Page already in MainMemory ppn=%d, returning FALSE\n", te.physicalPage);
    return FALSE;
  }
  /** page not in phys mem. 
  *** check in swap file
  *** if the swap file has it, report error
  *** if found, make enough room in phys mem
  *** Look if a free page available in MemoryManager::freePageList 
  **/
  if(swapt->Find(index, &se) ) {
    //just like ipt
    //printf("Page already in SwapMemory file-offset=%d, returning FALSE\n", se.position);
    return FALSE;
  }
  if(!kernel->memoryManager->freePageList->IsEmpty()){
    //if there is roon available on freePageList, why did you call this routine?
    //printf("Free room already available on freePageList\n");
    return FALSE;
  }
  /** 
  ***    GetFallGuy
  ***    Exchange fallguy for this page
  ***/

  TEWithPid toDisk;
  IPTIndex fgi;
  SwapEntry fgs;
  if(!GetFallGuy(&toDisk)){return FALSE;}
  fgi.vpn = toDisk.virtualPage;
  fgi.pid = toDisk.pid;
  AddrSpace* fallGuySpace = kernel->pidManager->getThread(toDisk.pid)->space;
  fallGuySpace->m_PageTable->Remove(toDisk.virtualPage);
  fgs.vpn = toDisk.virtualPage;
  fgs.proc = toDisk.pid;
  fgs.position = GetFreePosition();
  swapt->Insert(fgs);
  //printf("IPTManager::FreePageInMemory(...) FallGuy pid=%d, vpn=%d being saved to swap at %d\n", fgi.pid, fgi.vpn, fgs.position);
  if(WriteBlockAt(fgs)){;//This looks into ipt to get fileoffset
      //if this page is in TLB, you need to invalidate this entry
      //this is a problem only if the fallguy also belongs to this process
      if(pid == fallGuySpace->PID()){
        TranslationEntry tlbi;
        for(int i = 0; i < TLBSize; i++){
  	  tlbi = kernel->machine->tlb[i];
  	  if(tlbi.virtualPage == toDisk.virtualPage){
  	    kernel->machine->tlb[i].valid = FALSE;
  	    break;
  	  }
        }
      }
    ipt->Remove(fgi);//this must be done after write
    *ppnPtr = toDisk.physicalPage;
    //need to add it ipt
    te.pid = pid;
    te.virtualPage = vpn;
    te.physicalPage = *ppnPtr;
    te.valid = TRUE;
    ipt->Insert(te);
    //update process pageTable
    addrsp->m_PageTable->Enter(vpn,  *ppnPtr);
    //printf("Returning ppn=%d\n", *ppnPtr);
    return TRUE;
  }
  else return FALSE;
/**********************************
** dummy implementation
**********************************/
  return FALSE;
}
void IPTManager::MarkDirty(int proc, int vpn)
{
/**********************************
** dummy implementation
**********************************/
}
TEWithPid IPTManager::Remove(IPTIndex index) 
{ 
  /*****************************************************
  ** if this page is dirty you must write it to disk
  ** you may need to remove the page from the process pageTable
  ** correspondingly, you'll have to modify methods in PageTable
  ** to talk to MEmoryManager/IPTMAnager to get missing pages
  *****************************************************/
  TEWithPid retval; 
  if(swapt->IsInTable(index)) {
    /************************************************************
    ** i don't care for return value here as this is won't be 
    ** in any process' page table
    ** but i need to add this to freelist
    ************************************************************/
    SwapEntry se = swapt->Remove(index);
    freeBlocksList->Insert(se.position);
  }

  if(ipt->IsInTable(index)) {
    retval = ipt->Remove(index); 
    return retval;
  }
}
void IPTManager::Remove(int proc)
{
  HashIterator<IPTIndex, TEWithPid> iter(ipt);
  List<IPTIndex>* sl = new List<IPTIndex>;
  TEWithPid te;
  IPTIndex ipti;
  /*Iterator may behave funny if youdelete the item it is pointing to  */
  for (; !iter.IsDone(); iter.Next()) {
    te = iter.Item();
    if(te.pid != proc)continue;
    IPTIndex ipti(te.virtualPage, te.pid);
    sl->Append(ipti);
  }
  while(!sl->IsEmpty()){
   ipti = sl->RemoveFront();
   ipt->Remove(ipti);
  }

  HashIterator<IPTIndex, SwapEntry> iter1(swapt);
  List<IPTIndex>* sl1 = new List<IPTIndex>;
  //TEWithPid te;
  SwapEntry se;
  /*Iterator may behave funny if youdelete the item it is pointing to  */
  for (; !iter1.IsDone(); iter1.Next()) {
    se = iter1.Item();
    if(se.proc != proc)continue;
    IPTIndex i(se.vpn, se.proc);
    sl1->Append(i);
  }
  while(!sl1->IsEmpty()){
   ipti = sl1->RemoveFront();
   se = swapt->Remove(ipti);
   freeBlocksList->Insert(se.position);
  }

}
int IPTManager::GetFreePosition()
{
  if(freeBlocksList->IsEmpty()) {
    //printf("In IPTManager::GetFreePosition() - no free block in swap - writing at the end\n");
    int i = maxpos;
    maxpos += PageSize;
    return i;
  }
  int i = freeBlocksList->RemoveFront();
  //printf("In IPTManager::GetFreePosition() - returning free block %d\n", i);
  return i;
}

bool IPTManager::WriteBlockAt(SwapEntry se)
{
  IPTIndex index(se.vpn, se.proc);
  TEWithPid retval;
  if(ipt->IsInTable(index)) {
    ipt->Find(index, &retval); 
    char* dest = (char*) & kernel->machine->mainMemory[PageSize*retval.physicalPage];
    //int pos = GetFreePosition();
    int pos = se.position;
    swapFd->WriteAt(dest, PageSize, pos);
    return TRUE;
  }
  else return FALSE;

}
bool IPTManager::ReadBlockAt(SwapEntry se)
{
  IPTIndex index(se.vpn, se.proc);
  TEWithPid retval;
  if(ipt->IsInTable(index)) {
    ipt->Find(index, &retval); 
    char* src = (char*) & kernel->machine->mainMemory[PageSize*retval.physicalPage];
    //int pos = GetFreePosition();
    int pos = se.position;
    swapFd->ReadAt(src, PageSize, pos);
    return TRUE;
  }
  else return FALSE;

}
bool IPTManager::GetFallGuy(TEWithPid* tePtr)
{
  HashIterator<IPTIndex, TEWithPid> iter(ipt);
  /*Iterator may behave funny if youdelete the item it is pointing to  */
  if (!tePtr)return FALSE;
  for (; !iter.IsDone(); iter.Next()) {
    *tePtr = iter.Item();
    if (tePtr->use) continue;
    if (tePtr)return TRUE;
  }
  return FALSE;
}
#endif /* P3_1 */

