#ifdef P3_1
#ifndef IPTMANAGER_H
#define IPTMANAGER_H

#include "hash.h"
#include "translate.h"
#include "pidmanager.h"
#include "openfile.h"

#ifdef P4_1
#include "directory.h"
#define SWAPFILE "nachos.swap"
#else
#define SWAPFILE "./nachos.swap"
#endif

//We need to implement IPT for VM implementation
//The class that provides IPT is called IPTManager
//It contains IPT implemented as a hash table
//The key used for the hash table is IPTIndex
//class IPTIndex in turn contains PID & VirtPageNum
//The value corresponding to the key is a TranslationEntry
//The class HashTable requires us to be able to calculate the key for a given value
//Unfortunately, class TranslationEntry does not contain pid
//so you cannot construct an IPTIndex object from a TranslationEntry object
//So we have extended this class to TEWithPid
//Thus complete knowledge of phys page assignment across the board 
//for all the processes can be maintained.
//this obviates the need to rebuild IPT on context switch

class IPTIndex {

 public:

  IPTIndex(int v, int p) {
    vpn = v;
    pid = p;
  }

  IPTIndex() {
    //better this than unintialized values
    vpn = -1;
    pid = 0;
  }

  ~IPTIndex(){}

  bool operator==(IPTIndex& ipt) {
    return (this->pid == ipt.pid && this->vpn == ipt.vpn);
  }

  bool operator!=(IPTIndex& ipt) {
    return (this->pid != ipt.pid || this->vpn != ipt.vpn);
  }

  int vpn;
  int pid;

};

class TEWithPid: public TranslationEntry{

 public:

  TEWithPid(){
    pid = 0; 
    virtualPage = physicalPage = -1; 
    valid = readOnly = use = FALSE;
  }

  bool operator==(TEWithPid& te) {
    return (te.pid == this->pid && te.virtualPage == this->virtualPage);
  }

  bool operator!=(TEWithPid& te) {
    return (te.pid != this->pid || te.virtualPage != this->virtualPage);
  }

  int pid;

};

class SwapEntry {
 public:
  SwapEntry(int vpage, int pid, int where){vpn = vpage; proc = pid; position = where;}
  SwapEntry(){vpn = -1; proc = 0; position = -1;}
  bool operator==(SwapEntry& se){return (se.vpn == this->vpn && se.proc == this->proc);}
  bool operator!=(SwapEntry& se){return (se.vpn != this->vpn || se.proc != this->proc);}
  int vpn;
  int proc;
  int position;/*in the units of pages*/
};

class IPTManager {
public:

#ifdef P3_2
  IPTManager(MemoryManager* mmgrPtr);
#else
  IPTManager();
#endif

  ~IPTManager();
  bool HandleTlbMiss(int proc, int vpn, TEWithPid *tePtr);
  bool LoadVPInMemory(AddrSpace* addrsp, int vpn, int *ppnPtr);
  bool FreePageInMemory(AddrSpace* addrsp, int vpn, int *ppnPtr);/*called by MemeoryManager*/
  void Add(TEWithPid te){ipt->Insert(te);}/*called by MemeoryManager*/
  /************************************
  ** Remove(...) moethods are called by MemeoryManager when proc dies
  ** Remove(IPTIndex ...) removes the page from mainMem/swap
  ** and returns the info to the callee
  ** of course, if the page is in swap file, physpagenum will be bogus and set to -1

  ** Remove(int) removes all the pages owned by the pid
  ** irrespective of whether they are on mainMem or swap
  ** if they are on mainMem, ipt is modified
  ** if they are on Swap, then swapt, freeBlocksList & swapFd are modified
  **************************************/
  TEWithPid Remove(IPTIndex index);
  void Remove(int proc);/*called by MemoryManager when proc dies*/
  void MarkDirty(int proc, int vpn);
  bool Exchange(TEWithPid *toDisk, TEWithPid* fromDisk);
  bool GetFallGuy(TEWithPid* tePtr);
  bool WriteBlockAt(SwapEntry se);
  bool ReadBlockAt(SwapEntry se);
  int GetFreePosition();
  HashTable<IPTIndex, TEWithPid> *ipt;
  HashTable<IPTIndex, SwapEntry> *swapt;
  SortedList<int> *freeBlocksList;/*used to identify fee blocks on swap file*/
  bool Constructed() { return constructed; }
  MemoryManager* mmgr;
private:
  OpenFile* swapFd;
  int maxpos;/*in the units of pages*/
  bool constructed;
#ifdef P4_1
  Directory *swapdir;
#endif
};
#endif /* IPTMANAGER_H */
#endif /* P3_1 */
