#ifndef PIDMANAGER_H
#define PIDMANAGER_H

#include "syscall.h"
#include "synch.h"
#include "hash.h"

#ifdef P3_1
#include "thread.h"
#endif

typedef int PID;

const PID NOPID = 0;
const int MAX_ACTIVE_PIDS = 20;		// maximum # of process id that can
					// be 'active' (i.e., not zombie).
					// this implicitly implements a limit
					// on the number of processes in the
					// system.

class Lock;
class Condition;

enum PidState { IN_USE, ZOMBIE };

class PidInfo {
 public:
  PidInfo() {} ;
  ~PidInfo() {} ;

  PidState state;
  PID pid;  
#ifdef P3_1
  Thread *thread;
#endif
};

class PidManager {
public:
	PidManager(void);
	~PidManager(void);

	PID  PidAllocate(Thread *t);       /* allocates a PID for the process */
	void PidTerminate(PID pid);        /* marks the PID as ZOMBIE */
	int  PidWait(PID pid);             /* waits for PID to finish */
	int  ActivePids();                 /* return the # of active
	                                      pids (running procs) */
#ifdef P3_1
	Thread * getThread(PID pid);       /* return the thread
					      corresponding to PID */
#endif

private:
	const PID MAXPID = 100000;
	Lock *mutex;
	Condition *condition;
	HashTable<PID,PidInfo *> *procTable;
	PID NextPid;
	int ActivePidCount;
};

#endif








