// userkernel.cc 
//	Initialization and cleanup routines for the version of the
//	Nachos kernel that supports running user programs.
//
// Copyright (c) 1992-1996 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.
//

#include "copyright.h"
#include "synchconsole.h"
#include "userkernel.h"
#include "utility.h"

//----------------------------------------------------------------------
// UserProgKernel::UserProgKernel
// 	Interpret command line arguments in order to determine flags 
//	for the initialization (see also comments in main.cc)  
//----------------------------------------------------------------------

UserProgKernel::UserProgKernel(int argc, char **argv) 
		: ThreadedKernel(argc, argv)
{
    debugUserProg = FALSE;
    for (int i = 0; i < argc; i++) {
        if (strcmp(argv[i], "-s") == 0) {
	    debugUserProg = TRUE;
        } else if (strcmp(argv[i], "-u") == 0) {
            printf("Partial usage: nachos [-s]\n");
	}
    }
}

//----------------------------------------------------------------------
// UserProgKernel::Initialize
// 	Initialize Nachos global data structures.
//----------------------------------------------------------------------

void
UserProgKernel::Initialize()
{
    ThreadedKernel::Initialize();	// init multithreading

    consoleInput = new SynchConsoleInput(NULL);
    consoleOutput = new SynchConsoleOutput(NULL);

    machine = new Machine(debugUserProg);

#ifdef P2_2
    memoryManager = new MemoryManager;
    pidManager = new PidManager;
#endif

}

//----------------------------------------------------------------------
// UserProgKernel::~UserProgKernel
// 	Nachos is halting.  De-allocate global data structures.
//	Automatically calls destructor on base class.
//----------------------------------------------------------------------

UserProgKernel::~UserProgKernel()
{
    delete machine;

#ifdef P2_2
    delete memoryManager;
    delete pidManager;
#endif

}

static void LaunchUser(void*  name)
{

#ifdef P3_2
    AddrSpace *halt = new AddrSpace(kernel->currentThread->pid);
#else
    AddrSpace *halt = new AddrSpace();
#endif

    halt->Execute((char*)name);
}

//----------------------------------------------------------------------
// UserProgKernel::Run
// 	Run the Nachos kernel.  For now, just run the "halt" program. 
//----------------------------------------------------------------------

void
UserProgKernel::Run()
{

  //SelfTest();

#ifdef P2_2
#ifdef P3_1
#ifdef P4_1
    char* userprog = "/bin/shell";    
#else
    char* userprog = "./shell";    
#endif
#else
    char* userprog = "../test/SUN/exec";
#endif
    Thread *t1 = new Thread("UserShellThread",TRUE);

#ifdef P3_1
    t1->pid = kernel->pidManager->PidAllocate(t1);
#else
    t1->pid = kernel->pidManager->PidAllocate();
#endif

#ifndef P4_1_FOO
    t1->fileio = new FileIO(kernel->fileSystem->m_RootDirectory);
    t1->Fork((VoidFunctionPtr)LaunchUser, (void*)userprog);
    t1->Join();
    //while(t1)currentThread->Yield();
#endif
#else

    AddrSpace *halt = new AddrSpace();
#ifdef FILESYS
    halt->Execute("halt");
#else
#ifdef SIMOS
    halt->Execute("../test/SIMOS/halt");
#else
    halt->Execute("../test/SUN/halt");
#endif
#endif // FILESYS
#endif /* P2_2 */

    ThreadedKernel::Run();

}

//----------------------------------------------------------------------
// UserProgKernel::SelfTest
//      Test whether this module is working.
//----------------------------------------------------------------------

void
UserProgKernel::SelfTest() {


    char ch;

#ifndef P2_2
    ThreadedKernel::SelfTest();

    // test out the console device

    printf("Testing the console device.\n" 
           "Typed characters will be echoed, until q is typed.\n"
           "Note newlines are needed to flush the input line.\n");
#ifndef SIMOS
    fflush(stdout);
#endif
#endif /* ! P2_2 */

    do {
    	ch = consoleInput->GetChar();
    	consoleOutput->PutChar(ch);   // echo it!
    } while (ch != 'q');

    printf("\n");

    // self test for running user programs is to run the halt program above

}

